package com.snappay.pay.demo.alipay;

import android.annotation.SuppressLint;
import android.app.Activity;
import android.app.AlertDialog;
import android.app.ProgressDialog;
import android.content.Context;
import android.content.DialogInterface;
import android.os.Handler;
import android.os.Message;
import android.util.Log;

import com.alipay.sdk.app.PayTask;
import com.loopj.android.http.AsyncHttpClient;
import com.loopj.android.http.BaseJsonHttpResponseHandler;
import com.loopj.android.http.ResponseHandlerInterface;
import com.loopj.android.http.SyncHttpClient;
import com.snappay.pay.demo.R;
import com.snappay.pay.demo.utils.Constants;
import com.snappay.pay.demo.utils.PaymentUtil;

import org.json.JSONArray;
import org.json.JSONObject;

import java.util.Map;

import cz.msebera.android.httpclient.Header;
/**
 * 1.0 订单串 - AlipayGlobal 境外收单支付 本地签名逻辑
 * 注意：本 Demo 仅作为展示用途，实际项目中不能将 RSA_PRIVATE 和签名逻辑放在客户端进行！
 * New Cross-border APP Payment ： https://global-pre.site.alipay.net/service/app_split/1
 */
public final class AliPay {

    private AliPayOrder appOrder;
    private Activity activity;
    public ProgressDialog progressDlg;

    private static final int PAY_DONE = 1;
    private static final int PAY_SUCCESS = 2;
    private static final int PAY_FAILURE = 3;
    private static final int SYS_ERR = 4;

    @SuppressLint("HandlerLeak")
    private Handler mHandler = new Handler() {
        public void handleMessage(Message msg) {
            switch (msg.what) {
                case PAY_DONE:
                    String paymentStatus ="error";
                    checkPaymentStatus();
                    break;
                case PAY_SUCCESS:
                    dismisProcessingMessage();
                    showAlert(activity, "Payment is successful!", null);
                    break;
                case PAY_FAILURE:
                    dismisProcessingMessage();
                    showAlert(activity, "Payment is failure!", null);
                    break;
                case SYS_ERR:
                    dismisProcessingMessage();
                    showAlert(activity, "System error!", null);
                    break;
                default:
                    break;
            }
        };
    };

    public AliPay(Activity activity) {
        appOrder = new AliPayOrder();
        this.activity = activity;
    }

    public void executeAliPay() {
        processingMessage("Please wait......", false);

        final Runnable payRunnable = new Runnable() {

            @Override
            public void run() {
                AsyncHttpClient client = new SyncHttpClient();
                client.post(Constants.SNAPPAY_URL, PaymentUtil.buildCreateOrderParams("ALIPAY"), getCreateOrderResponseHandler());
            }
        };

        // 必须异步调用
        Thread payThread = new Thread(payRunnable);
        payThread.start();

    }

    public void checkPaymentStatus() {
        processingMessage("Checking payment status......", false);


        final Runnable payRunnable = new Runnable() {

            @Override
            public void run() {
                AsyncHttpClient client = new SyncHttpClient();
                try {
                    client.post(Constants.SNAPPAY_URL, PaymentUtil.buildCheckPaymentParams(appOrder.getTradeNo()), getCheckPaymentResponseHandler());
                }catch(Exception e){
                    e.printStackTrace();
                }
            }
        };

        // 必须异步调用
        Thread payThread = new Thread(payRunnable);
        payThread.start();
    }


    private ResponseHandlerInterface getCreateOrderResponseHandler() {
        return new BaseJsonHttpResponseHandler<JSONObject>() {

            @Override
            public void onStart() {
            }

            @Override
            public void onSuccess(int statusCode, Header[] headers, String rawJsonResponse, JSONObject response) {
                String test = rawJsonResponse;
                try {
                    JSONArray data = (JSONArray)response.get("data");
                    String callStatus = (String)response.get("code");
                    if ("0".equals(callStatus)) {
                        JSONObject sdkJson = (JSONObject)((JSONObject) data.get(0)).get("sdk_params");
                        String aliPayParams = sdkJson.getString("alipay_sdk_params");
                        String tradeNo = ((JSONObject) data.get(0)).get("trans_no").toString();
                        appOrder.setOrderInfo(aliPayParams);
                        appOrder.setTradeNo(tradeNo);
                        System.out.println(aliPayParams);
                        dismisProcessingMessage();
                        payOnAliPay();
                    } else {
                        dismisProcessingMessage();
                        showAlert(activity, "System error", null);
                    }
                } catch (Exception e) {
                    e.printStackTrace();
                }
                System.out.println(test);
            }

            @Override
            public void onFailure(int statusCode, Header[] headers, Throwable throwable, String rawJsonData, JSONObject errorResponse) {
                dismisProcessingMessage();
                Message msg = new Message();
                msg.what = SYS_ERR;
                msg.obj = null;
                mHandler.sendMessage(msg);
            }

            @Override
            protected JSONObject parseResponse(String rawJsonData, boolean isFailure) throws Throwable {
                JSONObject test = new JSONObject(rawJsonData);
                return test;
            }

        };
    }

    private ResponseHandlerInterface getCheckPaymentResponseHandler() {
        return new BaseJsonHttpResponseHandler<JSONObject>() {

            @Override
            public void onStart() {
            }

            @Override
            public void onSuccess(int statusCode, Header[] headers, String rawJsonResponse, JSONObject response) {
                String test = rawJsonResponse;
                try {
                    JSONArray data = (JSONArray)response.get("data");
                    String paymentStatus = ((JSONObject) data.get(0)).get("trans_status").toString();

                    Message msg = new Message();
                    msg.what = AliPay.PAY_FAILURE;
                    if ("SUCCESS".equals(paymentStatus)) {
                        msg.what = AliPay.PAY_SUCCESS;
                    }

                    msg.obj = paymentStatus;
                    mHandler.sendMessage(msg);

                    System.out.println(paymentStatus);
                } catch (Exception e) {
                    e.printStackTrace();
                }

                System.out.println(test);
            }

            @Override
            public void onFailure(int statusCode, Header[] headers, Throwable throwable, String rawJsonData, JSONObject errorResponse) {
                dismisProcessingMessage();
                Message msg = new Message();
                msg.what = SYS_ERR;
                msg.obj = null;
                mHandler.sendMessage(msg);
            }

            @Override
            protected JSONObject parseResponse(String rawJsonData, boolean isFailure) throws Throwable {
                JSONObject test = new JSONObject(rawJsonData);
                return test;
            }

            // ----New Overridden method
            @Override
            public boolean getUseSynchronousMode() {
                return false;
            }
        };
    }

    public void processingMessage(String message, boolean cancelable) {
        if (this.activity == null && message == null) {
            return;
        }
        progressDlg = new ProgressDialog(this.activity);

        progressDlg.setMessage(message);
        progressDlg.setIndeterminate(true);
        progressDlg.setCancelable(cancelable);
        progressDlg.show();
    }

    public void dismisProcessingMessage() {
        if (progressDlg != null) {
            progressDlg.dismiss();
        }
    }

    /**
     * Cross_border payment solution
     * 境外收单支付示例
     */
    public void payOnAliPay() {

		final Runnable payRunnable = new Runnable() {

			@Override
			public void run() {
				// 构造PayTask 对象
				PayTask alipay = new PayTask( AliPay.this.activity );
				// 调用支付接口，获取支付结果
				Map<String, String> result = alipay.payV2( AliPay.this.appOrder.getOrderInfo(), true );
				Log.i("msp", result.toString());

				Message msg = new Message();
				msg.what = PAY_DONE;
				msg.obj = result;
				mHandler.sendMessage(msg);
			}
		};

		// 必须异步调用
		Thread payThread = new Thread(payRunnable);
		payThread.start();
    }

    private static void showAlert(Context ctx, String info, DialogInterface.OnDismissListener onDismiss) {
        new AlertDialog.Builder(ctx)
                .setMessage(info)
                .setPositiveButton(R.string.confirm, null)
                .setOnDismissListener(onDismiss)
                .show();
    }
}
