package com.snappay.pay.demo.utils;

import com.loopj.android.http.RequestParams;
import com.snappay.pay.demo.SignUtils;

import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.Collections;
import java.util.Date;
import java.util.HashMap;
import java.util.List;
import java.util.Locale;
import java.util.Map;
import java.util.Random;

public class PaymentUtil {
    public static RequestParams buildCreateOrderParams(String paymentMethod) {

        Map<String, String> params = new HashMap<>();

        params.put("app_id",Constants.APP_ID);
        params.put("format","JSON");
        params.put("charset","UTF-8");
        params.put("version","1.0");
        params.put("method","pay.inapppay");
        params.put("trans_currency","CAD");
        params.put("refer_url","www.snappay.ca");
        params.put("merchant_no",Constants.MERCHANT_NO);
        params.put("payment_method",paymentMethod);
        params.put("out_order_no",getOutTradeNo());
        params.put("trans_amount","0.01");
        params.put("subject","Test Item");
        params.put("description","Test Item");
        params.put("effective_minutes","10");


        params.put("sign",signMD5(params));
        params.put("sign_type","MD5");

        RequestParams requestParams = new RequestParams(params);
        return requestParams;
    }

    /**
     * get the out_trade_no for an order. 生成商户订单号，该值在商户端应保持唯一（可自定义格式规范）
     *
     */
    private static String getOutTradeNo() {
        SimpleDateFormat format = new SimpleDateFormat("MMddHHmmss", Locale.getDefault());
        Date date = new Date();
        String key = format.format(date);

        Random r = new Random();
        key = key + r.nextInt();
        key = key.substring(0, 15);
        return "WY"+key;
    }

    private static String signMD5(Map<String, String> params) {
        return SignUtils.md5sign(createLinkString(params), Constants.MD5_KEY);
    }

    /**
     * 把数组所有元素排序，并按照“参数=参数值”的模式用“&”字符拼接成字符串
     * @param params 需要排序并参与字符拼接的参数组
     * @return 拼接后字符串
     */
    private static String createLinkString(Map<String, String> params) {
        List<String> keys = new ArrayList<String>(params.keySet());
        Collections.sort(keys);

        String prestr = "";

        for (int i = 0; i < keys.size(); i++) {
            String key = keys.get(i);
            String value = params.get(key);

            if (i == keys.size() - 1) {//拼接时，不包括最后一个&字符
                prestr = prestr + key + "=" + value;
            } else {
                prestr = prestr + key + "=" + value + "&";
            }
        }

        return prestr;
    }

    public static RequestParams buildCheckPaymentParams(String transNo) {
        Map<String, String> params = new HashMap<>();

        params.put("app_id",Constants.APP_ID);
        params.put("format","JSON");
        params.put("charset","UTF-8");
        params.put("version","1.0");

        params.put("method","pay.orderquery");
        params.put("merchant_no",Constants.MERCHANT_NO);
        params.put("trans_no",transNo);


        params.put("sign",signMD5(params));
        params.put("sign_type","MD5");

        RequestParams requestParams = new RequestParams(params);
        return requestParams;

    }
}

