package com.snappay.pay.demo.wechat;

import android.app.Activity;
import android.app.ProgressDialog;
import android.os.Message;

import com.loopj.android.http.AsyncHttpClient;
import com.loopj.android.http.BaseJsonHttpResponseHandler;
import com.loopj.android.http.ResponseHandlerInterface;
import com.loopj.android.http.SyncHttpClient;
import com.snappay.pay.demo.alipay.AliPayOrder;
import com.snappay.pay.demo.utils.Constants;
import com.snappay.pay.demo.utils.PaymentUtil;
import com.tencent.mm.opensdk.modelpay.PayReq;
import com.tencent.mm.opensdk.openapi.IWXAPI;
import com.tencent.mm.opensdk.openapi.WXAPIFactory;

import org.json.JSONArray;
import org.json.JSONObject;

import java.util.HashMap;
import java.util.Map;

import cz.msebera.android.httpclient.Header;

public class WeChatPay {

    private AliPayOrder appOrder;

    private static final int PAY_DONE = 1;
    private static final int PAY_SUCCESS = 2;
    private static final int PAY_FAILURE = 3;
    private static final int SYS_ERR = 4;
    private Activity activity;
    private IWXAPI msgApi;
    public ProgressDialog progressDlg;
    private static Map<String, String> prepayIdTransMapping = new HashMap<>();

    public WeChatPay( Activity activity ) {
        this.activity = activity;
        // 将该app注册到微信
        msgApi = WXAPIFactory.createWXAPI(this.activity, null);
        msgApi.registerApp(Constants.WX_APP_ID);
    }

    public void executeWeChatPay() {

        processingMessage("Please wait......", false);

        final Runnable payRunnable = new Runnable() {

            @Override
            public void run() {
                AsyncHttpClient client = new SyncHttpClient();
                client.post(Constants.SNAPPAY_URL, PaymentUtil.buildCreateOrderParams("WECHAT"), getCreateOrderResponseHandler());
            }
        };

        // 必须异步调用
        Thread payThread = new Thread(payRunnable);
        payThread.start();
    }

    public void processingMessage(String message, boolean cancelable) {
        if (this.activity == null && message == null) {
            return;
        }
        progressDlg = new ProgressDialog(this.activity);

        progressDlg.setMessage(message);
        progressDlg.setIndeterminate(true);
        progressDlg.setCancelable(cancelable);
        progressDlg.show();
    }

    public void dismisProcessingMessage() {
        if (progressDlg != null) {
            progressDlg.dismiss();
        }
    }

    public static String getTransByPrepayId(String prepayId) {
        return prepayIdTransMapping.get(prepayId);
    }

    public static void setTransByPrepayId(String prepayId, String transNo) {
        prepayIdTransMapping.put(prepayId, transNo);
    }

    private ResponseHandlerInterface getCreateOrderResponseHandler() {
        return new BaseJsonHttpResponseHandler<JSONObject>() {

            @Override
            public void onStart() {
            }

            @Override
            public void onSuccess(int statusCode, Header[] headers, String rawJsonResponse, JSONObject response) {
                String createResp = rawJsonResponse;
                try {
                    JSONArray data = (JSONArray)response.get("data");
                    String callStatus = (String)response.get("code");
                    if ("0".equals(callStatus)) {
                        JSONObject sdkJson = (JSONObject)((JSONObject) data.get(0)).get("sdk_params");

                        PayReq request = new PayReq();
                        request.appId = sdkJson.getString("appid");
                        request.partnerId = sdkJson.getString("partnerid");
                        request.prepayId= sdkJson.getString("prepayid");
                        request.packageValue = sdkJson.getString("package");
                        request.nonceStr= sdkJson.getString("noncestr");
                        request.timeStamp= sdkJson.getString("timestamp");
                        request.sign= sdkJson.getString("sign");
                        setTransByPrepayId(sdkJson.getString("prepayid"), ((JSONObject)data.get(0)).getString("trans_no"));
                        msgApi.sendReq(request);
                        dismisProcessingMessage();
                    } else {
                        dismisProcessingMessage();
                    }
                } catch (Exception e) {
                    e.printStackTrace();
                }
            }

            @Override
            public void onFailure(int statusCode, Header[] headers, Throwable throwable, String rawJsonData, JSONObject errorResponse) {
                dismisProcessingMessage();
                Message msg = new Message();
                msg.what = SYS_ERR;
                msg.obj = null;
            }

            @Override
            protected JSONObject parseResponse(String rawJsonData, boolean isFailure) throws Throwable {
                JSONObject test = new JSONObject(rawJsonData);
                return test;
            }

        };
    }
}
