package com.snappay.pay.demo.wxapi;

import android.annotation.SuppressLint;
import android.app.AlertDialog;
import android.app.ProgressDialog;
import android.content.Context;
import android.content.DialogInterface;
import android.content.Intent;
import android.os.Bundle;
import android.os.Handler;
import android.os.Message;
import android.support.v7.app.AppCompatActivity;
import android.widget.Toast;

import com.loopj.android.http.AsyncHttpClient;
import com.loopj.android.http.BaseJsonHttpResponseHandler;
import com.loopj.android.http.ResponseHandlerInterface;
import com.loopj.android.http.SyncHttpClient;
import com.snappay.pay.demo.R;
import com.snappay.pay.demo.utils.Constants;
import com.snappay.pay.demo.utils.PaymentUtil;
import com.snappay.pay.demo.wechat.WeChatPay;
import com.tencent.mm.opensdk.constants.ConstantsAPI;
import com.tencent.mm.opensdk.modelbase.BaseReq;
import com.tencent.mm.opensdk.modelbase.BaseResp;
import com.tencent.mm.opensdk.modelpay.PayResp;
import com.tencent.mm.opensdk.openapi.IWXAPI;
import com.tencent.mm.opensdk.openapi.IWXAPIEventHandler;
import com.tencent.mm.opensdk.openapi.WXAPIFactory;
import com.tencent.mm.opensdk.utils.Log;

import org.json.JSONArray;
import org.json.JSONObject;

import cz.msebera.android.httpclient.Header;

public class WXPayEntryActivity
        extends AppCompatActivity
implements IWXAPIEventHandler {
    private static final String TAG = "MicroMsg.SnapPay.WXPayEntryActivity";
    public ProgressDialog progressDlg;
    private IWXAPI api;

    private static final int PAY_DONE = 1;
    private static final int PAY_SUCCESS = 2;
    private static final int PAY_FAILURE = 3;
    private static final int SYS_ERR = 4;

    @SuppressLint("HandlerLeak")
    private Handler mHandler = new Handler() {
        public void handleMessage(Message msg) {
            switch (msg.what) {
                case PAY_SUCCESS:
                    dismisProcessingMessage();
                    showAlert(WXPayEntryActivity.this, "Payment is successful!", null);
                    break;
                case PAY_FAILURE:
                    dismisProcessingMessage();
                    showAlert(WXPayEntryActivity.this, "Payment is failure!", null);
                    break;
                case SYS_ERR:
                    dismisProcessingMessage();
                    showAlert(WXPayEntryActivity.this, "System error!", null);
                    break;
                default:
                    break;
            }
        };
    };

    public void onCreate(Bundle paramBundle) {
        super.onCreate(paramBundle);
        setContentView(R.layout.pay_result);
        api = WXAPIFactory.createWXAPI(this, null);
        api.registerApp(Constants.WX_APP_ID);
        api.handleIntent(getIntent(), this);
    }

    @Override
    protected void onNewIntent(Intent intent) {
        super.onNewIntent(intent);
        setIntent(intent);
        api.handleIntent(intent, this);
    }

    public void onReq(BaseReq paramBaseReq) {}

    public void onResp(BaseResp resp) {
        Log.d("resp:", "code=" + resp.errCode + ",str=" + resp.errStr);
        if (resp.getType() == ConstantsAPI.COMMAND_PAY_BY_WX) {
            switch (resp.errCode) {
                case 0: {
                    checkPaymentStatus(((PayResp)resp).prepayId);
                    break;
                }
                case -1: {
                    Toast.makeText(WXPayEntryActivity.this, "发生错误\t可能的原因：签名错误、未注册APPID、项目设置APPID不正确、注册的APPID与设置的不匹配、其他异常等。", Toast.LENGTH_SHORT).show();
                    break;
                }
                case -2: {
                    Toast.makeText(WXPayEntryActivity.this, "用户取消支付", Toast.LENGTH_SHORT).show();
                    break;
                }
            }
        }
    }

    public void checkPaymentStatus(final String prepayId) {
        processingMessage("Checking payment status......", false);


        final Runnable payRunnable = new Runnable() {

            @Override
            public void run() {
                AsyncHttpClient client = new SyncHttpClient();
                try {
                    client.post(Constants.SNAPPAY_URL, PaymentUtil.buildCheckPaymentParams(WeChatPay.getTransByPrepayId(prepayId)), getCheckPaymentResponseHandler());
                }catch(Exception e){
                    e.printStackTrace();
                }
            }
        };

        // 必须异步调用
        Thread payThread = new Thread(payRunnable);
        payThread.start();
    }

    public void processingMessage(String message, boolean cancelable) {
        if (this == null && message == null) {
            return;
        }
        progressDlg = new ProgressDialog(this);

        progressDlg.setMessage(message);
        progressDlg.setIndeterminate(true);
        progressDlg.setCancelable(cancelable);
        progressDlg.show();
    }

    public void dismisProcessingMessage() {
        if (progressDlg != null) {
            progressDlg.dismiss();
        }
    }

    private ResponseHandlerInterface getCheckPaymentResponseHandler() {
        return new BaseJsonHttpResponseHandler<JSONObject>() {

            @Override
            public void onStart() {
            }

            @Override
            public void onSuccess(int statusCode, Header[] headers, String rawJsonResponse, JSONObject response) {
                String result = rawJsonResponse;
                try {
                    JSONArray data = (JSONArray)response.get("data");
                    String paymentStatus = ((JSONObject) data.get(0)).get("trans_status").toString();
                    dismisProcessingMessage();
                    if ("SUCCESS".equals(paymentStatus)) {
                        Message msg = new Message();
                        msg.what = PAY_SUCCESS;
                        msg.obj = result;
                        mHandler.sendMessage(msg);
                    } else {
                        Message msg = new Message();
                        msg.what = PAY_FAILURE;
                        msg.obj = result;
                        mHandler.sendMessage(msg);
                    }

                } catch (Exception e) {
                    e.printStackTrace();
                }

            }

            @Override
            public void onFailure(int statusCode, Header[] headers, Throwable throwable, String rawJsonData, JSONObject errorResponse) {
                Message msg = new Message();
                msg.what = SYS_ERR;
                msg.obj = null;
                mHandler.sendMessage(msg);
            }

            @Override
            protected JSONObject parseResponse(String rawJsonData, boolean isFailure) throws Throwable {
                JSONObject test = new JSONObject(rawJsonData);
                return test;
            }

            // ----New Overridden method
            @Override
            public boolean getUseSynchronousMode() {
                return false;
            }
        };
    }

    private static void showAlert(Context ctx, String info, DialogInterface.OnDismissListener onDismiss) {
        new AlertDialog.Builder(ctx)
                .setMessage(info)
                .setPositiveButton(R.string.confirm, null)
                .setOnDismissListener(onDismiss)
                .show();
    }
}
